String Version = "V0.3 AS338/AS361";
/*
 * Firmware für AS338 und AS361 
 *  
 * DL2MAJ, 12-Jan-2021
 * 
 * Zeitinformation kommt von DCF77-RX (Conrad-Modul 641138) -> D2
 * LDR liefert Helligkeit -> A0
 * DS18B20 liefern Innen- bzw. Aussentemperatur -> D3 
 *  
 * Ausgabe auf 7-Segmentanzeige via MAX7221 (#2) -> D10, D9, D8
 * Ausgabe Sekunden auf LED-Kreis via MAX7221 (#1) -> D7, D6, D5
 * 
 * serial monitor : 38k4
 */

#include <DCF77.h>
#include <LedControl.h>
#include <Time.h>
#include <TimeLib.h>
#include <OneWire.h>
#include <DallasTemperature.h>

#define DCF_PIN 2                // Connection pin to DCF 77 device
#define DCF_INTERRUPT 0          // Interrupt number associated with pin

time_t time;
DCF77 DCF = DCF77(DCF_PIN,DCF_INTERRUPT);

// Variablendeklaration
int Stunde = 0; // aktuelle Stunde
byte HZ = 0;     // Stunden-Zehner
byte HE = 0;     // Stunden-Einer
int alteStunde = 0;   // letzte Stunde
byte Minuten = 0;      // aktuelle Minute
int Hilf;   // Hilfsvariable
byte MZ = 0;     // Minuten-Zehner
byte ME = 0;     // Minuten-Einer
int alteMinute = 0;   // letzte Minute
int Sekunde = 0;      // aktuelle Sekunde
int alteSekunde = 0;  // letzte Sekunde
int Tag = 0;    // aktueller Tag
int alterTag = 0;     // letzter Tag;
byte TZ = 0;     // Tag-Zehner
byte TE = 1;     // Tag-Einer
int Monat = 0;  // aktueller Monat
int alterMonat = 0;   // letzter Monat
byte MoZ = 0;    // Monats-Zehner
byte MoE = 1;    // Monats-Einer
boolean TimeOK = false; // DCF77-Signal ausgewertet
boolean eineLED = false;    // false: je Sekunde wird eine LED eingeschaltet
                              // true : je Sekunde leuchtet nur 1 LED
int Zeile = 0;
int Spalte = 0;
int LDR = 0;      // Analog-Pin A0
int Helligkeit = 3;
int alteHelligkeit = 3;
int HellDiff = 0;
byte Leucht;      // Intensitaetsstufe fuer MAX7221
float InnenTemp;  // Innentemperatur mit erstem DS18B20
float AussenTemp;  // Aussentemperatur mit zweitem DS18B20

/* 
 * Now we create a new LedControl. 
 * We use pins 7, 6 and 5 on the Arduino for the SPI interface
 * Pin 7 is connected to the DATA IN-pin of the first MAX7221
 * Pin 6 is connected to the CLK-pin of the first MAX7221
 * Pin 5 is connected to the LOAD(/CS)-pin of the first MAX7221       
 * There will only be a single MAX7221 attached to the arduino 
 */
LedControl lc1=LedControl(7,6,5,1); // LEDs

/* 
 * Now we create a new LedControl. 
 * We use pins 10, 9 and 8 on the Arduino for the SPI interface
 * Pin 10 is connected to the DATA IN-pin of the first MAX7221
 * Pin 9 is connected to the CLK-pin of the first MAX7221
 * Pin 8 is connected to the LOAD(/CS)-pin of the first MAX7221       
 * There will only be a single MAX7221 attached to the arduino 
 */
LedControl lc2=LedControl(10,9,8,1); // 7-Segment

/********************************************************************/
// Data wire is plugged into pin 3 on the Arduino 
#define ONE_WIRE_BUS 3 
/********************************************************************/
// Setup a oneWire instance to communicate with any OneWire devices  
// (not just Maxim/Dallas temperature ICs) 
OneWire oneWire(ONE_WIRE_BUS); 
/********************************************************************/
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature EinWire(&oneWire);
/********************************************************************/ 

void setup() {
 // put your setup code here, to run once:
    // setup MAX7221
    // activate display on MAX7221
    lc1.shutdown(0,false);
    lc2.shutdown(0,false);

    //set an environment dependend brightness for the LEDs
    setzeHelligkeit();

    lc1.clearDisplay(0);
    lc2.clearDisplay(0);

    //configure D2 as input and enable the internal pull-up resistor
    pinMode(2, INPUT_PULLUP);  // Sekundenimpuls von DCF77-RX

    // Start up the OneWire-library 
    EinWire.begin(); 

    EinWire.requestTemperatures(); // Send the command to get temperatures

    delay(800);
      
    InnenTemp = EinWire.getTempCByIndex(0); 
    AussenTemp = EinWire.getTempCByIndex(1);
    // Why "byIndex"?  
    // You can have more than one DS18B20 on the same bus.  
    // 0 refers to the first IC on the wire
    
    // initialize the serial port:
    Serial.begin(38400);

    Serial.println(" ");
    Serial.print("Firmware-Version : ");
    Serial.println(Version);
    Serial.println(" ");

    Serial.print("InnenTemp original = ");
    Serial.println(InnenTemp);
    delay(200);
    subInnenTemp();

    Serial.print("AussenTemp original = ");
    Serial.println(AussenTemp);
    delay(200);
    subAussenTemp();

    Serial.println();
    delay(1000);
  
    lc2.setDigit(0,2,(byte)HZ,false);
    lc2.setDigit(0,3,(byte)HE,false);
    lc2.setDigit(0,0,(byte)MZ,false);
    lc2.setDigit(0,1,(byte)ME,false);

    lc2.setDigit(0,4,(byte)TZ,false);
    lc2.setDigit(0,5,(byte)TE,true);
    lc2.setDigit(0,6,(byte)MoZ,false);
    lc2.setDigit(0,7,(byte)MoE,true);

    // Entwanzen();
    
    // initialize the DCF77  
    DCF.Start();
    Serial.println("Waiting for DCF77 time ... ");
    Serial.println("It will take at least 2 minutes before a first time update.");

    do
    {
      delay(500);
      time_t DCFtime = DCF.getTime(); // Check if new DCF77 time is available
      if (DCFtime!=0)
      {
            Serial.println("Internal time is updated");
            setTime(DCFtime);
            TimeOK = true;
      }

      lc1.clearDisplay(0);
      ZeitUpdate();
      digitalClockDisplay(); 
    }   
    while (TimeOK == false);      
}


void loop() {
  // put your main code here, to run repeatedly:

  time_t DCFtime = DCF.getTime(); // Check if new DCF77 time is available
  if (DCFtime!=0)
  {
    Serial.println("Internal time is updated");
    setTime(DCFtime);
  }    

  ZeitUpdate();
}
// end of main program



void ZeitUpdate() {

    Stunde = hour();
    Minuten = minute();
    Sekunde = second();
    Tag = day();
    Monat = month();
  
    // Helligkeit einstellen
    setzeHelligkeit();

     if (Sekunde != alteSekunde)
    {
       // SekundenLEDs einstellen  
       subSekunden();
    }

     if (Minuten != alteMinute)
    {
      subMinute();
    }
   
    if (Stunde != alteStunde)
    {
      subStunde();
    }
    
     if (Tag != alterTag)
    {
      subTag();
    }
    
     if (Monat != alterMonat)
    {
      subMonat();
    }    

    alteStunde = Stunde;
    alteMinute = Minuten;
    alteSekunde = Sekunde;
    alterTag = Tag;
    alterMonat = Monat; 
}

void setzeHelligkeit()  {
    Helligkeit = analogRead(LDR);   // grosse Zahl = dunkel

    HellDiff = Helligkeit - alteHelligkeit;

    if (HellDiff < 0)   
    {
      HellDiff = -HellDiff;
    }

    if (HellDiff > 10)
    {
      if (Helligkeit < 130)
      {
        Leucht = 15;
      }
    
      else if (Helligkeit < 180)
        {
          Leucht = 14;
        }
    
        else if (Helligkeit < 230)
          {
            Leucht = 13;
          }
            
          else if (Helligkeit < 280)
            {
              Leucht = 12;
            }
      
            else if (Helligkeit < 330)
              {
                Leucht = 11;
              }
    
              else if (Helligkeit < 380)
                {
                  Leucht = 10;
                }
                  
                else if (Helligkeit < 430)
                  {
                    Leucht = 9;
                  }
                         
                  else if (Helligkeit < 480)
                    {
                      Leucht = 8;
                    }
    
                    else if (Helligkeit < 530)
                      {
                        Leucht = 7;
                      }
    
                      else if (Helligkeit < 580)
                        {
                          Leucht = 6;
                        }
            
                        else if (Helligkeit < 630)
                          {
                            Leucht = 5;
                          }
                      
                          else if (Helligkeit < 680)
                            {
                              Leucht = 4;
                            }

                            else if (Helligkeit < 730)
                              {
                                Leucht = 3;
                              }
    
                              else if (Helligkeit < 780)
                                {
                                  Leucht = 2;
                                }
    
                                else if (Helligkeit < 830)
                                  {
                                    Leucht = 1;
                                  }
    
                                  else 
                                    {
                                      Leucht = 0;                                      
                                    }
                                    
     if (Helligkeit > 880)   // sehr dunkel
     {
      eineLED = true;                      
     }
     else
     {
      eineLED = false;
     }
     
     lc1.setIntensity(0,Leucht);
     lc2.setIntensity(0,Leucht); 
  
/*     Serial.print("alteHelligkeit = ");   // debug only
     Serial.println(alteHelligkeit);
     Serial.print("Helligkeit = ");
     Serial.println(Helligkeit);
     Serial.print("Leucht = ");
     Serial.println(Leucht);
*/         
     alteHelligkeit = Helligkeit;     
    }
}
  
void digitalClockDisplay(){
  // digital clock display of the time
  Serial.print(hour());
  printDigits(minute());
  printDigits(second());
  Serial.print(" ");
  Serial.print(day());
  Serial.print(" ");
  Serial.print(month());
  Serial.print(" ");
  Serial.print(year());
  Serial.println();
}

void printDigits(int digits){
  // utility function for digital clock display: prints preceding colon and leading 0
  Serial.print(":");
  if(digits < 10)
    Serial.print('0');
  Serial.print(digits);
}

void HZ_HE() {
  // creates HZ und HE
  if (Stunde > 19) 
  {
      HZ = 2; 
      HE = Stunde - 20;
  }
  else if (Stunde < 10)  
  {
      HZ = 0;
      HE = Stunde;
  }
  else 
  {
      HE = Stunde - 10;
      HZ = 1;
  }
   
  Serial.print ("Stunden : ");
  Serial.print(HZ);
  Serial.print(" ");
  Serial.println(HE);
}
  
void MZ_ME() {
  // creates MZ und ME
  
  if (Minuten <10)
  {
      MZ = 0;
      ME = Minuten;
  }
  else
  {
      MZ = 0;
      Hilf = Minuten;
  
      do
      {
            MZ = MZ + 1;
            Hilf = Hilf - 10;
      }     while (Hilf > 9);
  
      ME = Hilf;
  }
  
  
  Serial.print ("Minuten : ");
  Serial.print(MZ);
  Serial.print(" ");
  Serial.println(ME);
}  

void TZ_TE() {
  // creates TZ und TE
  if (Tag <10)
  {
      TZ = 0;
      TE = Tag;
  }
  else
  {
      TZ = 0;
      Hilf = Tag;
  
      do
      {
            TZ = TZ + 1;
            Hilf = Hilf - 10;
      }     while (Hilf > 9);
  
      TE = Hilf;
  }
 
  Serial.print ("Tag : ");
  Serial.print(TZ);
  Serial.print(" ");
  Serial.println(TE);
}  

void MoZ_MoE() {
  // creates MoZ und MoE
  if (Monat > 9) 
  {
      MoZ = 1; 
      MoE = Monat - 10;
  }
  else
  {
      MoE = Monat;
      MoZ = 0;
  } 

  Serial.print ("Monat : ");
  Serial.print(MoZ);
  Serial.print(" ");
  Serial.println(MoE);
}

void subInnenTemp() {
  int Einer;
  int Zehner;
  int Hunderter;
  boolean negative = false;      
  long hilfsTemp;
  float hilfsTemp1;

  hilfsTemp1 = InnenTemp + 0.05;
  hilfsTemp1 = 10 * hilfsTemp1;
  hilfsTemp = hilfsTemp1;

    if(hilfsTemp < -999 || hilfsTemp > 999) 
       return;
       
    if(hilfsTemp < 0) 
    {
        negative=true;
        hilfsTemp = -hilfsTemp;
    }
    
    Einer=hilfsTemp%10;
    hilfsTemp=hilfsTemp/10;
    Zehner=hilfsTemp%10;
    hilfsTemp=hilfsTemp/10;
    Hunderter=hilfsTemp;               
    if(negative) {
       //print character '-' in the leftmost column   
       lc2.setChar(0,2,'-',false);
    }
    else {
       //print a blank in the sign column
       lc2.setChar(0,2,' ',false);
    }
    //Now print the number digit by digit
    lc2.setDigit(0,3,(byte)Hunderter,false);
    lc2.setDigit(0,0,(byte)Zehner,true);
    lc2.setDigit(0,1,(byte)Einer,false);

/*  Serial.print ("HilfsTemp1 : ");   // debug only
  Serial.println(hilfsTemp1);
  Serial.print ("Innentemp : ");
  Serial.print(Hunderter);
  Serial.print(Zehner);
  Serial.print(".");
  Serial.println(Einer);
*/
}

void subAussenTemp() {
  int Einer;
  int Zehner;
  int Hunderter;
  boolean negative = false;      
  long hilfsTemp;
  float hilfsTemp1;

  hilfsTemp1 = AussenTemp + 0.05;
  hilfsTemp1 = 10 * hilfsTemp1;
  hilfsTemp = hilfsTemp1;

    if(hilfsTemp < -999 || hilfsTemp > 999) 
       return;
       
    if(hilfsTemp < 0) 
    {
        negative=true;
        hilfsTemp = -hilfsTemp;
    }
    
    Einer=hilfsTemp%10;
    hilfsTemp=hilfsTemp/10;
    Zehner=hilfsTemp%10;
    hilfsTemp=hilfsTemp/10;
    Hunderter=hilfsTemp;               
    if(negative) {
       //print character '-' in the leftmost column   
       lc2.setChar(0,4,'-',false);
    }
    else {
       //print a blank in the sign column
       lc2.setChar(0,4,' ',false);
    }
    //Now print the number digit by digit
    lc2.setDigit(0,5,(byte)Hunderter,false);
    lc2.setDigit(0,6,(byte)Zehner,true);
    lc2.setDigit(0,7,(byte)Einer,false);   

/*  Serial.print ("HilfsTemp1 : ");   // debug only
  Serial.println(hilfsTemp1);
  Serial.print ("AussenTemp : ");
  Serial.print(Hunderter);
  Serial.print(Zehner);
  Serial.print(".");
  Serial.println(Einer);
*/
}

void subSekunden() 
{
int hilfssekunde;

// creates Zeile und Spalte
  Zeile = 0;
   
  hilfssekunde = Sekunde;
  
  if (hilfssekunde  < 8)
  {
      Spalte = hilfssekunde;
  }
  else
  {
      Hilf = hilfssekunde;
  
      do
      {
            Zeile = Zeile + 1;
            Hilf = Hilf - 8;
      }     while (Hilf > 7);
  
      Spalte = Hilf;
  }

  if (eineLED == 1)
  {
      lc1.clearDisplay(0);
  }
  
  lc1.setLed(0, Zeile, Spalte, true);

  switch (second() )
  {
      case 13:
            EinWire.requestTemperatures(); // Send the command to get temperatures
            break;
            
      case 14:
            InnenTemp = EinWire.getTempCByIndex(0);
            break;

      case 15:
            AussenTemp = EinWire.getTempCByIndex(1);
            break;
                  
      case 16:
            subInnenTemp();
            subAussenTemp();
            break;

      case 21:
            lc2.setDigit(0,2,(byte)HZ,false);
            lc2.setDigit(0,3,(byte)HE,false);
            lc2.setDigit(0,0,(byte)MZ,false);
            lc2.setDigit(0,1,(byte)ME,false);

            lc2.setDigit(0,4,(byte)TZ,false);
            lc2.setDigit(0,5,(byte)TE,true);
            lc2.setDigit(0,6,(byte)MoZ,false);
            lc2.setDigit(0,7,(byte)MoE,true);
            break;

      case 43:
            EinWire.requestTemperatures(); // Send the command to get temperatures
            break;
            
      case 44:
            InnenTemp = EinWire.getTempCByIndex(0);
            break;

      case 45:
            AussenTemp = EinWire.getTempCByIndex(1);
            break;
                  
      case 46:
            subInnenTemp();
            subAussenTemp();
            break;

      case 51:
            lc2.setDigit(0,2,(byte)HZ,false);
            lc2.setDigit(0,3,(byte)HE,false);
            lc2.setDigit(0,0,(byte)MZ,false);
            lc2.setDigit(0,1,(byte)ME,false);

            lc2.setDigit(0,4,(byte)TZ,false);
            lc2.setDigit(0,5,(byte)TE,true);
            lc2.setDigit(0,6,(byte)MoZ,false);
            lc2.setDigit(0,7,(byte)MoE,true);
            break;
  }
 
  if (hilfssekunde == 0)    // Wert fuer debug
  {
      Serial.print("Sek : ");
      Serial.print(hilfssekunde);
      if (hilfssekunde < 10)
      {
            Serial.print("     Zeile/Spalte : ");
      }
      else
      {
            Serial.print("    Zeile/Spalte : ");
      }
  
      Serial.print(Zeile);
      Serial.print("/");
      Serial.println(Spalte);    
  }
}  

void subStunde()  {
    HZ_HE();
    // Stundenanzeige einstellen  
    lc2.setDigit(0,2,(byte)HZ,false);
    lc2.setDigit(0,3,(byte)HE,false);
}

void subMinute()  {
    MZ_ME();
    // Minutenanzeige einstellen 
    lc1.clearDisplay(0); 
    lc1.setLed(0, Zeile, Spalte, true);
    lc2.setDigit(0,0,(byte)MZ,false);
    lc2.setDigit(0,1,(byte)ME,false);
}

void subTag()     {
    TZ_TE();
    // Taganzeige einstellen  
    lc2.setDigit(0,4,(byte)TZ,false);
    lc2.setDigit(0,5,(byte)TE,true);
}

void subMonat()   {
    MoZ_MoE();
    // Monatsanzeige einstellen  
    lc2.setDigit(0,6,(byte)MoZ,false);
    lc2.setDigit(0,7,(byte)MoE,true);
}

void Entwanzen() {

    for (int i=0; i <= 8; i++){
          lc2.setDigit(0,i,i,false);
          delay(100);
    }
      for (int i=0; i<1000; i++) {
            setzeHelligkeit();
  
            Serial.print("Helligkeit/Leucht : ");
            Serial.print(Helligkeit);
            Serial.print("/");
            Serial.println(Leucht);

            delay(100);
      }

    for (int i=0; i <= 8; i++){
      for (int j=0; j <= 9; j++) {
             lc2.setDigit(0,i,(byte)j,false);
          delay(1000);
      }
      delay(10);
   }   
}
